/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.SearchBean;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Vector;

import ca.uvic.csr.shrimp.SelectorBeanConstants;
import ca.uvic.csr.shrimp.DisplayBean.ShrimpNode;
import ca.uvic.csr.shrimp.SelectorBean.SelectorBean;
import ca.uvic.csr.shrimp.ShrimpApplication.BeanNotFoundException;
import ca.uvic.csr.shrimp.ShrimpApplication.ShrimpToolNotFoundException;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProjectAdapter;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProjectEvent;
import ca.uvic.csr.shrimp.ShrimpTool.ShrimpTool;
import ca.uvic.csr.shrimp.gui.ShrimpView.ShrimpView;
import ca.uvic.csr.shrimp.util.CollectionUtils;


/**
 * Links the {@link SearchTool} search textbox with the first selected node's name from {@link ShrimpView}.
 * 
 * @author Chris Callendar
 * @date 06-Nov-06
 */
public class ShrimpViewSearchToolBridge extends ShrimpProjectAdapter implements PropertyChangeListener {

	private SearchTool searchTool;

	public ShrimpViewSearchToolBridge(SearchTool searchTool) {
		this.searchTool = searchTool;
	}
	
	public void init() {
		try {
			ShrimpProject project = searchTool.getProject();
			if (project != null) {
				project.addProjectListener(this);
				ShrimpView shrimpView = (ShrimpView) project.getTool(ShrimpProject.SHRIMP_VIEW); 
				SelectorBean selectorBean = (SelectorBean) shrimpView.getBean(ShrimpTool.SELECTOR_BEAN);
				selectorBean.addPropertyChangeListener(SelectorBeanConstants.SELECTED_NODES, this);
				setSelectionToSearchTool(selectorBean);
			}
		} catch (Exception e) {
		}
	}
	
	public void removeListeners() {
		ShrimpProject project = searchTool.getProject();
		if (project != null) {
			project.removeProjectListener(this);
			try {
				ShrimpView shrimpView = (ShrimpView) project.getTool(ShrimpProject.SHRIMP_VIEW); 
				SelectorBean selectorBean = (SelectorBean) shrimpView.getBean(ShrimpTool.SELECTOR_BEAN);
				selectorBean.removePropertyChangeListener(SelectorBeanConstants.SELECTED_NODES, this);
			} catch (BeanNotFoundException e) {
			} catch (ShrimpToolNotFoundException e) {
			}
		}
	}
	
	public void projectClosing(ShrimpProjectEvent event) {
		event.getProject().removeProjectListener(this);
		removeListeners();
	}

	public void propertyChange(PropertyChangeEvent evt) {
		Vector oldSelected = (Vector) evt.getOldValue();
		Vector newSelected = (Vector) evt.getNewValue();
		if (!CollectionUtils.haveSameElements(oldSelected, newSelected)) {
			setSelectionToSearchTool(newSelected);
		}
	}

	public void setSelectionToSearchTool(SelectorBean selectorBean) {
		if ((searchTool != null) && (selectorBean != null)) {
			Vector selectedNodes = (Vector) selectorBean.getSelected(SelectorBeanConstants.SELECTED_NODES);
			setSelectionToSearchTool(selectedNodes);
		}
	}

	private void setSelectionToSearchTool(Vector selectedNodes) {
		// get the artifacts from the nodes
		String text = "";
		if (selectedNodes.size() > 0) {
			ShrimpNode node = (ShrimpNode) selectedNodes.get(0);
			text = node.getArtifact().getName();
		}

		searchTool.setSearchText(text);
	}

}
